#include <stdlib.h>
#include "construct_path.h"

/* constructs a new path */
path_t* new_path(command_list_list_t* cmds, float size) {
  path_t* path = (path_t*)malloc(sizeof(path_t));
  path->cmds = cmds;
  path->size = size;
  return path;
}

/* constructs a new list of lists of commands */
command_list_list_t* new_command_list_list(command_list_t* head) {
  command_list_list_t* list = (command_list_list_t*)malloc(sizeof(command_list_list_t));
  list->head = head;
  list->tail = head;
  return list;
}

/* constructs a new list of lists of commands */
command_list_list_t* new_empty_command_list_list() {
  command_list_list_t* list = (command_list_list_t*)malloc(sizeof(command_list_list_t));
  list->head = 0;
  list->tail = 0;
  return list;
}

/* constructs a new list of commands */
command_list_t* new_command_list(command_t* head) {
  command_list_t* list = (command_list_t*)malloc(sizeof(command_list_t));
  list->head = head;
  list->tail = head;
  list->next = 0;
  return list;
}

/* constructs a new list of commands */
command_list_t* new_empty_command_list() {
  command_list_t* list = (command_list_t*)malloc(sizeof(command_list_t));
  list->head = 0;
  list->tail = 0;
  list->next = 0;
  return list;
}

/* constructs a new coordinate pair command */
command_t* new_coord_pair_command(abs_or_rel_t abs_or_rel,
                                  cmd_type_t cmd_type,
                                  float x,
                                  float y) {
  command_t* cmd = (command_t*)malloc(sizeof(command_t));
  cmd->type = cmd_type;
  cmd->abs_or_rel = abs_or_rel;
  cmd->x = x;
  cmd->y = y;
  cmd->next = 0;
  return cmd;
}

/* constructs a new move command */
command_t* new_move_command(abs_or_rel_t abs_or_rel, float x, float y) {
  return new_coord_pair_command(abs_or_rel, MOVE_CMD, x, y);
}

/* constructs a new line command */
command_t* new_line_command(abs_or_rel_t abs_or_rel, float x, float y) {
  return new_coord_pair_command(abs_or_rel, LINE_CMD, x, y);
}

/* constructs a new hline command */
command_t* new_hline_command(abs_or_rel_t abs_or_rel, float x) {
  return new_coord_pair_command(abs_or_rel, HLINE_CMD, x, 0.0);
}

/* constructs a new vline command */
command_t* new_vline_command(abs_or_rel_t abs_or_rel, float y) {
  return new_coord_pair_command(abs_or_rel, VLINE_CMD, 0.0, y);
}

/* constructs a new cubic curve command */
command_t* new_cubic_curve_command(abs_or_rel_t abs_or_rel,
                                   float x1,
                                   float y1,
                                   float x2,
                                   float y2,
                                   float x,
                                   float y) {
    command_t* cmd = (command_t*)malloc(sizeof(command_t));
    cmd->type = CUBIC_CURVE_CMD;
    cmd->abs_or_rel = abs_or_rel;
    cmd->control_point_data.x1 = x1;
    cmd->control_point_data.y1 = y1;
    cmd->control_point_data.x2 = x2;
    cmd->control_point_data.y2 = y2;
    cmd->x = x;
    cmd->y = y;
    cmd->next = 0;
    return cmd;
}

/* constructs a new smooth cubic curve command */
command_t* new_smooth_cubic_curve_command(abs_or_rel_t abs_or_rel,
                                          float x2,
                                          float y2,
                                          float x,
                                          float y) {
    command_t* cmd = (command_t*)malloc(sizeof(command_t));
    cmd->type = SMOOTH_CUBIC_CURVE_CMD;
    cmd->abs_or_rel = abs_or_rel;
    cmd->control_point_data.x2 = x2;
    cmd->control_point_data.y2 = y2;
    cmd->x = x;
    cmd->y = y;
    cmd->next = 0;
    return cmd;
}

/* constructs a new quadratic curve command */
command_t* new_quad_curve_command(abs_or_rel_t abs_or_rel,
                                  float x1,
                                  float y1,
                                  float x,
                                  float y) {
    command_t* cmd = (command_t*)malloc(sizeof(command_t));
    cmd->type = QUAD_CURVE_CMD;
    cmd->abs_or_rel = abs_or_rel;
    cmd->control_point_data.x1 = x1;
    cmd->control_point_data.y1 = y1;
    cmd->x = x;
    cmd->y = y;
    cmd->next = 0;
    return cmd;
}

/* constructs a new smooth quadratic curve command */
command_t* new_smooth_quad_curve_command(abs_or_rel_t abs_or_rel,
                                         float x,
                                         float y) {
    command_t* cmd = (command_t*)malloc(sizeof(command_t));
    cmd->type = SMOOTH_QUAD_CURVE_CMD;
    cmd->abs_or_rel = abs_or_rel;
    cmd->x = x;
    cmd->y = y;
    cmd->next = 0;
    return cmd;
}

/* constructs a new arc command */
command_t* new_arc_command(abs_or_rel_t abs_or_rel,
                           float rx,
                           float ry,
                           float x_axis_rotation,
                           float large_arc_flag,
                           float sweep_flag,
                           float x,
                           float y) {
    command_t* cmd = (command_t*)malloc(sizeof(command_t));
    cmd->type = ARC_CMD;
    cmd->abs_or_rel = abs_or_rel;
    cmd->arc_data.rx = rx;
    cmd->arc_data.ry = ry;
    cmd->arc_data.x_axis_rotation = x_axis_rotation;
    cmd->arc_data.flags = (int)large_arc_flag | ((int)sweep_flag << 1);
    cmd->x = x;
    cmd->y = y;
    cmd->next = 0;
    return cmd;
}

/* constructs a new close path command */
command_t* new_close_path_command() {
    command_t* cmd = (command_t*)malloc(sizeof(command_t));
    cmd->type = CLOSE_PATH_CMD;
    cmd->next = 0;
    return cmd;
}

/* adds a command to the end of a command list */
void concat(command_list_t* list, command_t* next) {
  if (list->head == 0) {
    list->head = next;
    list->tail = next;
  } else {
    list->tail->next = next;
    list->tail = next;
  }
}

/* adds a command list to the end of a command list list */
void concat_lists(command_list_list_t* list, command_list_t* next) {
  if (list->head == 0) {
    list->head = next;
    list->tail = next;
  } else {
    list->tail->next = next;
    list->tail = next;
  }
}

/* walks the list and returns the length */
int list_length(command_list_t* list) {
  int n = 0;
  command_t* cur = list->head;
  while(cur) {
    n++;
    cur = cur->next;
  }
  return n;
}
