#ifndef _CONSTRUCT_PATH_H_
#define _CONSTRUCT_PATH_H_

#include "path_types.h"

/* constructs a new path */
path_t* new_path(command_list_list_t* cmds, float size);

/* constructs a new list of lists of commands */
command_list_list_t* new_command_list_list(command_list_t* head);

/* constructs a new list of lists of commands */
command_list_list_t* new_empty_command_list_list();

/* constructs a new list of commands */
command_list_t* new_command_list(command_t* head);

/* constructs a new list of commands */
command_list_t* new_empty_command_list();

/* constructs a new coordinate pair command */
command_t* new_coord_pair_command(abs_or_rel_t abs_or_rel,
                                  cmd_type_t cmd_type,
                                  float x,
                                  float y);

/* constructs a new move command */
command_t* new_move_command(abs_or_rel_t abs_or_rel, float x, float y);

/* constructs a new line command */
command_t* new_line_command(abs_or_rel_t abs_or_rel, float x, float y);

/* constructs a new hline command */
command_t* new_hline_command(abs_or_rel_t abs_or_rel, float x);

/* constructs a new vline command */
command_t* new_vline_command(abs_or_rel_t abs_or_rel, float y);

/* constructs a new cubic curve command */
command_t* new_cubic_curve_command(abs_or_rel_t abs_or_rel,
                                   float x1,
                                   float y1,
                                   float x2,
                                   float y2,
                                   float x,
                                   float y);

/* constructs a new smooth cubic curve command */
command_t* new_smooth_cubic_curve_command(abs_or_rel_t abs_or_rel,
                                          float x2,
                                          float y2,
                                          float x,
                                          float y);

/* constructs a new quadratic curve command */
command_t* new_quad_curve_command(abs_or_rel_t abs_or_rel,
                                  float x1,
                                  float y1,
                                  float x,
                                  float y);

/* constructs a new smooth quadratic curve command */
command_t* new_smooth_quad_curve_command(abs_or_rel_t abs_or_rel,
                                         float x,
                                         float y);

/* constructs a new arc command */
command_t* new_arc_command(abs_or_rel_t abs_or_rel,
                           float rx,
                           float ry,
                           float x_axis_rotation,
                           float large_arc_flag,
                           float sweep_flag,
                           float x,
                           float y);

/* constructs a new close path command */
command_t* new_close_path_command();

/* adds a command to the end of a command list */
void concat(command_list_t* list, command_t* next);

/* adds a command list to the end of a command list list */
void concat_lists(command_list_list_t* list, command_list_t* next);

/* walks the list and returns the length */
int list_length(command_list_t* list);

#endif
