///////////////////////////////////////
/// Lab 4 svg parser and laser control code
///////////////////////////////////////
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <sys/mman.h>
#include <sys/time.h>
#include <math.h>

#include <pthread.h>
#include <semaphore.h>
#include <string.h>

/* According to POSIX.1-2001, POSIX.1-2008 */
#include <sys/select.h>

//According to earlier standards
#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>

#include "process_path.h"
#include "svg_parser.h"

// ==========================================

/// lw_bus
#define HW_REGS_BASE           (0xff200000)
#define HW_REGS_SPAN           (0x00010000)

#define SOLVER_RESET_PIO_BASE  (0x00000010)
#define SOLVER_ENABLE_PIO_BASE (0x00000020)
#define SOLVER_DONE_PIO_BASE   (0x00000030)
#define TYPE_MEMORY_BASE       (0x00009000)
#define X0_MEMORY_BASE         (0x00008000)
#define X1_MEMORY_BASE         (0x00007000)
#define X2_MEMORY_BASE         (0x00006000)
#define X3_MEMORY_BASE         (0x00005000)
#define Y0_MEMORY_BASE         (0x00004000)
#define Y1_MEMORY_BASE         (0x00003000)
#define Y2_MEMORY_BASE         (0x00002000)
#define Y3_MEMORY_BASE         (0x00001000)

// the light weight bus base
void *h2p_lw_virtual_base;

volatile unsigned int* solver_reset_ptr  = NULL;
volatile unsigned int* solver_enable_ptr = NULL;
volatile unsigned int* solver_done_ptr   = NULL;
volatile unsigned int* type_memory_ptr   = NULL;
volatile unsigned int* x0_memory_ptr     = NULL;
volatile unsigned int* x1_memory_ptr     = NULL;
volatile unsigned int* x2_memory_ptr     = NULL;
volatile unsigned int* x3_memory_ptr     = NULL;
volatile unsigned int* y0_memory_ptr     = NULL;
volatile unsigned int* y1_memory_ptr     = NULL;
volatile unsigned int* y2_memory_ptr     = NULL;
volatile unsigned int* y3_memory_ptr     = NULL;

// /dev/mem file id
int fd;

// function declarations
void send_path_to_solver(command_list_t* path, char first_time);

///////////////////////////////////////////////////////////////

int main(int argc, char **argv)
{
  // === get FPGA addresses ==================
  // Open /dev/mem
  if( ( fd = open( "/dev/mem", ( O_RDWR | O_SYNC ) ) ) == -1 )   {
    printf( "ERROR: could not open \"/dev/mem\"...\n" );
    return( 1 );
  }
  
  // get virtual addr that maps to physical
  h2p_lw_virtual_base = mmap( NULL, HW_REGS_SPAN, ( PROT_READ | PROT_WRITE ), MAP_SHARED, fd, HW_REGS_BASE );
  if( h2p_lw_virtual_base == MAP_FAILED ) {
    printf( "ERROR: mmap() failed...\n" );
    close( fd );
    return(1);
  }
  
  solver_reset_ptr  = (unsigned int *)(h2p_lw_virtual_base + SOLVER_RESET_PIO_BASE);
  solver_enable_ptr = (unsigned int *)(h2p_lw_virtual_base + SOLVER_ENABLE_PIO_BASE);
  solver_done_ptr   = (unsigned int *)(h2p_lw_virtual_base + SOLVER_DONE_PIO_BASE);
  type_memory_ptr   = (unsigned int *)(h2p_lw_virtual_base + TYPE_MEMORY_BASE);
  x0_memory_ptr     = (unsigned int *)(h2p_lw_virtual_base + X0_MEMORY_BASE);
  x1_memory_ptr     = (unsigned int *)(h2p_lw_virtual_base + X1_MEMORY_BASE);
  x2_memory_ptr     = (unsigned int *)(h2p_lw_virtual_base + X2_MEMORY_BASE);
  x3_memory_ptr     = (unsigned int *)(h2p_lw_virtual_base + X3_MEMORY_BASE);
  y0_memory_ptr     = (unsigned int *)(h2p_lw_virtual_base + Y0_MEMORY_BASE);
  y1_memory_ptr     = (unsigned int *)(h2p_lw_virtual_base + Y1_MEMORY_BASE);
  y2_memory_ptr     = (unsigned int *)(h2p_lw_virtual_base + Y2_MEMORY_BASE);
  y3_memory_ptr     = (unsigned int *)(h2p_lw_virtual_base + Y3_MEMORY_BASE);

  // === Parse path file ========================================
  // Check svg file was passed as an argument
  ++argv, --argc;
  if ( argc == 0 ) {
    printf("usage: lab4 <svg_file>\n");
    exit(0);
  }
	
  int rotate_flag = 0;
  if (argc >= 2) {
    rotate_flag = atoi(argv[1]);
  }

  int fps = 10;
  if (argc >= 3) {
    fps = atoi(argv[2]);
  }

  float seconds_per_rotation = 3.0;
  if (argc >= 4) {
    seconds_per_rotation = atof(argv[3]);
  }

  // Parse svg file into path
  path_t* path = parse_svg(argv[0]);

  // === Process path file ========================================
  command_list_t* cmds = flatten(path->cmds);
  normalize(cmds);
  scale_and_close(cmds, path->size, 20.0);

  // === Write output ========================================
  // Open output file
  FILE* output = fopen("output", "w");
  if (output == NULL) {
    fprintf(stderr, "Could not open output file\n");
    exit(0);
  }

  // Write processed path to output
  print_path_full(output, cmds);
  fclose(output);

  // Send commands to solver memory
  send_path_to_solver(cmds, 1);
	
  if (rotate_flag) {
    // wait for a little while then rotate the path and send the next frame
    struct timeval start_time, cur_time;
    long usec;
    float theta;
		
    gettimeofday(&start_time, NULL);
    while(1) {
      // wait
      do {
        gettimeofday(&cur_time, NULL);
        usec = ((cur_time.tv_sec * 1000000 + cur_time.tv_usec)
        - (start_time.tv_sec * 1000000 + start_time.tv_usec));
      } while (usec < 1000000/fps);
      gettimeofday(&start_time, NULL);
		
      // calculate the angle and apply the rotation
      theta = 2.0*M_PI/seconds_per_rotation * usec / 1000000.0; 
      rotate(cmds, 127.5, 127.5, theta);
		
      // send the updated path to the fpga
      send_path_to_solver(cmds, 0);
    }
  }

} // end main

void send_path_to_solver(command_list_t* path, char first_time) {
  if (!first_time) {
    // Disable solver
    *solver_enable_ptr = 0;
    // Wait until the frame finishes
    while (*solver_done_ptr == 0);
  }
  // Hold solver in reset
  *solver_reset_ptr = 1;

  // Write path commands to solver memory
  command_t* cur = path->head;
  int i = 0;
  while(cur) {
    switch(cur->type) {
      case MOVE_CMD:
        type_memory_ptr[i] = SOLVER_MOVE;
        x0_memory_ptr[i]   = float2fix(cur->x0);
        y0_memory_ptr[i]   = float2fix(cur->y0);
        x1_memory_ptr[i]   = float2fix(cur->x);
        y1_memory_ptr[i]   = float2fix(cur->y);
        break;
      case LINE_CMD:
        type_memory_ptr[i] = SOLVER_LINE;
        x0_memory_ptr[i]   = float2fix(cur->x0);
        y0_memory_ptr[i]   = float2fix(cur->y0);
        x1_memory_ptr[i]   = float2fix(cur->x);
        y1_memory_ptr[i]   = float2fix(cur->y);
        break;
      case QUAD_CURVE_CMD:
        type_memory_ptr[i] = SOLVER_QUAD;
        x0_memory_ptr[i]   = float2fix(cur->x0);
        y0_memory_ptr[i]   = float2fix(cur->y0);
        x1_memory_ptr[i]   = float2fix(cur->control_point_data.x1);
        y1_memory_ptr[i]   = float2fix(cur->control_point_data.y1);
        x2_memory_ptr[i]   = float2fix(cur->x);
        y2_memory_ptr[i]   = float2fix(cur->y);
        break;
      case CUBIC_CURVE_CMD:
        type_memory_ptr[i] = SOLVER_CUBIC;
        x0_memory_ptr[i]   = float2fix(cur->x0);
        y0_memory_ptr[i]   = float2fix(cur->y0);
        x1_memory_ptr[i]   = float2fix(cur->control_point_data.x1);
        y1_memory_ptr[i]   = float2fix(cur->control_point_data.y1);
        x2_memory_ptr[i]   = float2fix(cur->control_point_data.x2);
        y2_memory_ptr[i]   = float2fix(cur->control_point_data.y2);
        x3_memory_ptr[i]   = float2fix(cur->x);
        y3_memory_ptr[i]   = float2fix(cur->y);
        break;
      default:
        fprintf(stderr, "Unexpected %c command after normalization\n", cmd_to_char(cur));
        exit(0);
        break;
    }
    i++;
    cur = cur->next;
  }

  // Append path end command
  type_memory_ptr[i] = SOLVER_END;

  // Release solver from reset
  *solver_reset_ptr = 0;
  // Enable solver
  *solver_enable_ptr = 1;
}
