%{
#include <stdlib.h>
#include <stdio.h>
#include "construct_path.h"

extern int yylex();
command_list_list_t* parse_tree;

void yyerror(const char *s) {
  fprintf (stderr, "%s\n", s);
}
%}
/*Tokens declarations*/

%union {
  float num;
  abs_or_rel_t abs_or_rel;
  command_list_t* cmd_list;
  command_list_list_t* cmd_list_list;
}

/* Command tokens */
%token <abs_or_rel> MOVE
%token <abs_or_rel> LINE
%token <abs_or_rel> HLINE
%token <abs_or_rel> VLINE
%token <abs_or_rel> CUBIC_CURVE
%token <abs_or_rel> SMOOTH_CUBIC_CURVE
%token <abs_or_rel> QUAD_CURVE
%token <abs_or_rel> SMOOTH_QUAD_CURVE
%token <abs_or_rel> ARC
%token CLOSE_PATH

/* Data tokens */
%token <num> NUM

/* Nonterminal type declarations */
%type<cmd_list_list> svg_path
%type<cmd_list> path_cmd move_cmd line_cmd hline_cmd vline_cmd
%type<cmd_list> cubic_curve_cmd smooth_cubic_curve_cmd
%type<cmd_list> quad_curve_cmd smooth_quad_curve_cmd
%type<cmd_list> arc_cmd close_path_cmd

%%
/*Rules Declarations*/
svg_path:
  move_cmd            { $$ = new_command_list_list($1); parse_tree = $$; }
  | svg_path path_cmd { concat_lists($$ = $1, $2); }

path_cmd:
  move_cmd
  | line_cmd
  | hline_cmd
  | vline_cmd
  | cubic_curve_cmd
  | smooth_cubic_curve_cmd
  | quad_curve_cmd
  | smooth_quad_curve_cmd
  | arc_cmd
  | close_path_cmd

move_cmd:
  MOVE NUM NUM
  { $$ = new_command_list(new_move_command($1, $2, $3)); }
  | move_cmd NUM NUM
  { concat($$ = $1, new_move_command($1->head->abs_or_rel, $2, $3)); }

line_cmd:
  LINE NUM NUM
  { $$ = new_command_list(new_line_command($1, $2, $3)); }
  | line_cmd NUM NUM
  { concat($$ = $1, new_line_command($1->head->abs_or_rel, $2, $3)); }

hline_cmd:
  HLINE NUM
  { $$ = new_command_list(new_hline_command($1, $2)); }
  | hline_cmd NUM
  { concat($$ = $1, new_hline_command($1->head->abs_or_rel, $2)); }

vline_cmd:
  VLINE NUM
  { $$ = new_command_list(new_vline_command($1, $2)); }
  | vline_cmd NUM
  { concat($$ = $1, new_vline_command($1->head->abs_or_rel, $2)); }

cubic_curve_cmd:
  CUBIC_CURVE NUM NUM NUM NUM NUM NUM
  { $$ = new_command_list(new_cubic_curve_command($1, $2, $3, $4, $5, $6, $7)); }
  | cubic_curve_cmd NUM NUM NUM NUM NUM NUM
  { concat($$ = $1, new_cubic_curve_command($1->head->abs_or_rel, $2, $3, $4, $5, $6, $7)); }

smooth_cubic_curve_cmd:
  SMOOTH_CUBIC_CURVE NUM NUM NUM NUM
  { $$ = new_command_list(new_smooth_cubic_curve_command($1, $2, $3, $4, $5)); }
  | smooth_cubic_curve_cmd NUM NUM NUM NUM
  { concat($1, new_smooth_cubic_curve_command($1->head->abs_or_rel, $2, $3, $4, $5)); }

quad_curve_cmd:
  QUAD_CURVE NUM NUM NUM NUM
  { $$ = new_command_list(new_quad_curve_command($1, $2, $3, $4, $5)); }
  | quad_curve_cmd NUM NUM NUM NUM
  { concat($$ = $1, new_quad_curve_command($1->head->abs_or_rel, $2, $3, $4, $5)); }

smooth_quad_curve_cmd:
  SMOOTH_QUAD_CURVE NUM NUM
  { $$ = new_command_list(new_smooth_quad_curve_command($1, $2, $3)); }
  | smooth_quad_curve_cmd NUM NUM NUM NUM
  { concat($$ = $1, new_smooth_quad_curve_command($1->head->abs_or_rel, $2, $3)); }

arc_cmd:
  ARC NUM NUM NUM NUM NUM NUM NUM
  { $$ = new_command_list(new_arc_command($1, $2, $3, $4, $5, $6, $7, $8)); }
  | arc_cmd NUM NUM NUM NUM NUM NUM NUM
  { concat($$ = $1, new_arc_command($1->head->abs_or_rel, $2, $3, $4, $5, $6, $7, $8)); }

close_path_cmd:
  CLOSE_PATH { $$ = new_command_list(new_close_path_command()); }

%%
