#ifndef _PROCESS_PATH_H_
#define _PROCESS_PATH_H_

#include <stdio.h>
#include "path_types.h"

// Solver path segment type encodings
#define SOLVER_MOVE  (0)
#define SOLVER_LINE  (1)
#define SOLVER_QUAD  (2)
#define SOLVER_CUBIC (3)
#define SOLVER_END   (4)

// conversion to 10.18 fixed point format
#define float2fix(a) ((signed int)((a)*pow(2,18)))

/* Flattens a list of lists into a single list */
command_list_t* flatten(command_list_list_t* path);

/* Walks a list of commands and applies the following changes:
 * - transforms all commands into absolute coordinates
 * - populates the x0 and y0 start point fields for each command
 * - transforms hline and vline commands into equivalent line commands
 * - transforms smooth bezier curves into their non shorthand equivalents
 * - transforms close path commands into their equivalent line commands
 */
void normalize(command_list_t* path);

/* Walks a list of commands and applies the following changes:
 * - scales all coordinates from the space [0,original_size] to the space [offset,255-offset]
 * - makes the path begin and end with moves from and to the center
 */
void scale_and_close(command_list_t* path, float original_size, float offset);

/* Walks a list of commands and rotates all points by theta around point (cx,cy) */
void rotate(command_list_t* path, float cx, float cy, float theta);

/* Converts a command to its character encoding */
char cmd_to_char(command_t* cmd);

/* Prints a list of commands in svg path format to a file */
void print_path_svg(FILE* output, command_list_t* path);

/* Prints a list of commands in longhand format to a file */
void print_path_full(FILE* output, command_list_t* path);

/* Prints a list of commands in hdl format to a file */
void print_path_hdl(FILE* output, command_list_t* path);

#endif
