///////////////////////////////////////
/// Toggle reset
///////////////////////////////////////
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <sys/mman.h>
#include <sys/time.h>
#include <math.h>

#include <pthread.h>
#include <semaphore.h>
#include <string.h>

/* According to POSIX.1-2001, POSIX.1-2008 */
#include <sys/select.h>

//According to earlier standards
#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>

// conversion to 10.18 fixed point format
#define float2fix(a) ((signed int)((a)*pow(2,18)))

// ==========================================
/// lw_bus
#define HW_REGS_BASE          (0xff200000)
#define HW_REGS_SPAN          (0x00010000)

#define SOLVER_RESET_PIO_BASE (0x00000010)
#define TYPE_MEMORY_BASE      (0x00009000)
#define X0_MEMORY_BASE        (0x00008000)
#define X1_MEMORY_BASE        (0x00007000)
#define X2_MEMORY_BASE        (0x00006000)
#define X3_MEMORY_BASE        (0x00005000)
#define Y0_MEMORY_BASE        (0x00004000)
#define Y1_MEMORY_BASE        (0x00003000)
#define Y2_MEMORY_BASE        (0x00002000)
#define Y3_MEMORY_BASE        (0x00001000)

// the light weight bus base
void *h2p_lw_virtual_base;

volatile unsigned int* solver_reset_ptr = NULL;
volatile unsigned int* type_memory_ptr  = NULL;
volatile unsigned int* x0_memory_ptr    = NULL;
volatile unsigned int* x1_memory_ptr    = NULL;
volatile unsigned int* x2_memory_ptr    = NULL;
volatile unsigned int* x3_memory_ptr    = NULL;
volatile unsigned int* y0_memory_ptr    = NULL;
volatile unsigned int* y1_memory_ptr    = NULL;
volatile unsigned int* y2_memory_ptr    = NULL;
volatile unsigned int* y3_memory_ptr    = NULL;

// /dev/mem file id
int fd;

///////////////////////////////////////////////////////////////

int main(int argc, char **argv)
{
  // === get FPGA addresses ==================
  // Open /dev/mem
  if( ( fd = open( "/dev/mem", ( O_RDWR | O_SYNC ) ) ) == -1 )   {
    printf( "ERROR: could not open \"/dev/mem\"...\n" );
    return( 1 );
  }
  
  // get virtual addr that maps to physical
  h2p_lw_virtual_base = mmap( NULL, HW_REGS_SPAN, ( PROT_READ | PROT_WRITE ), MAP_SHARED, fd, HW_REGS_BASE );
  if( h2p_lw_virtual_base == MAP_FAILED ) {
    printf( "ERROR: mmap() failed...\n" );
    close( fd );
    return(1);
  }
  
  solver_reset_ptr = (unsigned int *)(h2p_lw_virtual_base + SOLVER_RESET_PIO_BASE);
  type_memory_ptr  = (unsigned int *)(h2p_lw_virtual_base + TYPE_MEMORY_BASE);
  x0_memory_ptr    = (unsigned int *)(h2p_lw_virtual_base + X0_MEMORY_BASE);
  x1_memory_ptr    = (unsigned int *)(h2p_lw_virtual_base + X1_MEMORY_BASE);
  x2_memory_ptr    = (unsigned int *)(h2p_lw_virtual_base + X2_MEMORY_BASE);
  x3_memory_ptr    = (unsigned int *)(h2p_lw_virtual_base + X3_MEMORY_BASE);
  y0_memory_ptr    = (unsigned int *)(h2p_lw_virtual_base + Y0_MEMORY_BASE);
  y1_memory_ptr    = (unsigned int *)(h2p_lw_virtual_base + Y1_MEMORY_BASE);
  y2_memory_ptr    = (unsigned int *)(h2p_lw_virtual_base + Y2_MEMORY_BASE);
  y3_memory_ptr    = (unsigned int *)(h2p_lw_virtual_base + Y3_MEMORY_BASE);

  // === Parse path file ========================================
  // Check path file was passed as an argument
  ++argv, --argc;
  if ( argc == 0 ) {
    printf("usage: reset <0|1>\n");
    exit(0);
  }
  
  *solver_reset_ptr = atoi(argv[0]);

} // end main
