#include <stdio.h>
#include <libxml/parser.h>
#include <libxml/tree.h>

#include "construct_path.h"
#include "process_path.h"

extern command_list_list_t* parse_tree;
extern int yyparse();

typedef struct yy_buffer_state * YY_BUFFER_STATE;
extern YY_BUFFER_STATE yy_scan_string(const char * str);
extern int yylex_destroy();

void parse_paths(command_list_list_t* cmds, xmlNode* node) {
  xmlNode* cur = node;

  while(cur) {
    if(xmlStrcmp(cur->name, (const xmlChar*)"path") == 0) {
      char* path_data = (char*)xmlGetProp(cur, (const xmlChar*)"d");
      yy_scan_string(path_data);
      int rv  = yyparse();
      if (rv > 0) {
        fprintf(stderr, "Choked parsing path %s\n", path_data);
        exit(0);
      }
      yylex_destroy();
      free(path_data);

      command_list_t* cmd = flatten(parse_tree);
      concat_lists(cmds, cmd);
    }
    parse_paths(cmds, cur->children);
    cur = cur->next;
  }
}

path_t* parse_svg(const char* name) {
  xmlDoc *doc = NULL;
  xmlNode *cur = NULL;

  doc = xmlParseFile(name);

  if (doc == NULL) {
    fprintf(stderr, "Failed to parse svg file %s\n", name);
    exit(0);
  }

  cur = xmlDocGetRootElement(doc);

  if (cur == NULL) {
    fprintf(stderr, "svg file %s is empty\n", name);
    exit(0);
  }
  else if (xmlStrcmp(cur->name, (const xmlChar*)"svg") != 0) {
    fprintf(stderr, "Root node of file %s is not svg\n", name);
    exit(0);
  }

  float height, width;
  char* height_str = (char*)xmlGetProp(cur, (const xmlChar*)"height");
  char* width_str = (char*)xmlGetProp(cur, (const xmlChar*)"width");
  if (height_str) {
    height = atof(height_str);
    free(height_str);
  }
  else {
    height = 255.0;
  }
  if (width_str) {
    width = atof(width_str);
    free(width_str);
  }
  else {
    width = 255.0;
  }
  float size = (height > width) ? (height) : (width);

  command_list_list_t* cmds = new_empty_command_list_list();
  parse_paths(cmds, cur);

  if (cmds->head == NULL) {
    fprintf(stderr, "No paths in svg file %s\n", name);
    exit(0);
  }

  return new_path(cmds, size);
}
